/*
 * VitamioB4A is B4A-Wrapper of the Vitamio-plugin (Copyright (C) 2011VOV IO - http://vov.io/)
 * 
 * This B4A-Wrapper is open-source and must remain so. This code is the source of VitamioB4A i.e.
 * the B4A-Wrapper of the Vitamio-plugin.
 * Original author of the VitamioB4A-wrapper: moster67 (info@tillekesoft.com)
 * First date of publication: July 5, 2012 at the Basic4Android-forum
 * Version: 1.05 of July 14, 2012
 *  
 * Licenses (information from http://vov.io/vitamio/): 
 * 
 * Vitamio uses several open-source projects, thanks to them.
 * FFmpeg:
 * FFmpeg provides the software codecs and demuxers. Vitamio uses code of FFmpeg (www.http://ffmpeg.org/) 
 * licensed under the LGPLv2.1 (http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html) 
 * and its source can be downloaded here (http://d.vov.io/vitamio/ffmpeg.7z) 
 * Universalchardet:
 * Universalchardet is the encoding detector library of Mozilla. Vitamio uses the code of universalchardet 
 * licensed under MPL to detect the encoding of subtitle texts. The original code of universalchardet 
 * is available at Mozilla
 * (http://lxr.mozilla.org/seamonkey/source/extensions/universalchardet/ - note: On July 5, 2012 link not working).
 * The code of universalchardet used by Vitamio is available here 
 * (http://d.vov.io/vitamio/universalchardet.7z. - note: On July 5, 2012 link not working) 
 * 
 */

package forum.b4a.vitamio;


import io.vov.vitamio.widget.VideoView;
import io.vov.vitamio.MediaPlayer;
import io.vov.vitamio.MediaPlayer.OnCompletionListener;
import io.vov.vitamio.MediaPlayer.OnErrorListener;
import io.vov.vitamio.MediaPlayer.OnPreparedListener;
import io.vov.vitamio.MediaPlayer.OnInfoListener;
import io.vov.vitamio.MediaPlayer.OnBufferingUpdateListener;
import io.vov.vitamio.widget.MediaController;
import android.net.Uri;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.ActivityObject;
import anywheresoftware.b4a.BA.Author;
import anywheresoftware.b4a.BA.DependsOn;
import anywheresoftware.b4a.BA.Permissions;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;
import anywheresoftware.b4a.objects.ViewWrapper;



//import android.util.Log;



/**
 * VitamioB4A is a B4A-Wrapper of the Vitamio-plugin. The plugin is an Android APK which 
 * includes a collection of media demuxers, decoders, filters besides some video and audio 
 * output libraries. If the user does not have the plugin installed, the user will be prompted 
 * to download and install it from Google Play. Since the first release of the B4A-wrapper, many
 * other features have been added and now only subtitles-features of the VideoView-class are missing.
 * Other developers are encouraged to improve this first release of VitamioB4A including bug-fixes, 
 * adding other classes and methods/properties.
 * 
 * Note: android:minSdkVersion="8" 
 * 
 * VitamioB4A allows you to play video media inside your applications. 
 * The difference from the standard  Android-VideoView is that VitamioB4A supports much more formats 
 * and codecs since it is based on FFMPEG.
 * The VideoView shows a media controller (disabled by default) when the user touches 
 * the view. The Completed event is raised when playback is completed.
 * Simple example of using VitamioB4A VideoView:<code>
     *Sub Globals
     *    Dim vv As VideoView
     *End Sub
     *Sub Activity_Create(FirstTime As Boolean)
     *    vv.Initialize("vv")
     *    Activity.AddView(vv, 10dip, 10dip, 250dip, 250dip)
     *    vv.LoadVideo(File.DirRootExternal, "somefile.mp4")
     *End Sub
     *Sub vv_Complete
     *    Log("Playing completed")
     *End Sub</code>
 * Note: The method Play is not needed since it starts automatically.
 * Note2: To obtain maximum quality and to avoid pixeling, you must use the method 
 * VideoQuality and set it to 16. For reasons beyond my competence, I cannot set 
 * VideoQuality when using it within the method LoadVideo. The only way I got it 
 * working was by using a timer (see sample-app).
 * Suggestion: If you add the VideoView to a panel, you can implement nice stuff 
 * such OSD (On Screen Display) and other functionality.
 *
 *
 */

@ActivityObject
@Version(1.05F)
@Author("moster67")
@Permissions(values={"android.permission.INTERNET"})
@DependsOn(values="vitamio")
@ShortName("VideoView")
public class vitamiowrap extends ViewWrapper<VideoView> {
	
	//private static final String TAG = "MyActivity";
	
	VideoView vitamioExt=null;
	
	
	private int vidQuality=16; 
	private int mLayout = VideoView.VIDEO_LAYOUT_ZOOM;
	
	 public void Initialize(final BA ba, final String EventName)
	  {
	   vitamioExt=new VideoView(ba.context);
	   
	   
	  
	    super.setObject(vitamioExt);
	    super.Initialize(ba, EventName);
	    
	    vitamioExt.setOnCompletionListener(new OnCompletionListener() {
			@Override
			public void onCompletion(MediaPlayer mp) {
				ba.raiseEvent(((VideoView)getObject()), EventName + "_complete", new Object[0]);
			}
		});
	    
	   vitamioExt.setOnErrorListener(new OnErrorListener() {			
			@Override
			public boolean onError(MediaPlayer mp, int what, int extra) {
				ba.raiseEvent(((VideoView)getObject()), EventName + "_error", new Object[] { Integer.valueOf(what), Integer.valueOf(extra) });
				//return false; //enables vitamio's pop-up error messages
				return true;
			}
		});
	    	
	    	vitamioExt.setOnPreparedListener(new OnPreparedListener(){			
				@Override
				public void onPrepared(MediaPlayer mp) {
					ba.raiseEvent(((VideoView)getObject()), EventName + "_prepared", new Object[0]);
				}
			});
	    	
	    	vitamioExt.setOnInfoListener(new OnInfoListener() {			
				@Override
				public boolean onInfo(MediaPlayer mp, int what, int extra)  {
					ba.raiseEvent(((VideoView)getObject()), EventName + "_info", new Object[] { Integer.valueOf(what), Integer.valueOf(extra) });
					//return false;
					return true;
				}
			});
	    	
	    	vitamioExt.setOnBufferingUpdateListener(new OnBufferingUpdateListener() {			
				@Override
				public void onBufferingUpdate(MediaPlayer mp, int percent)  {
					ba.raiseEvent(((VideoView)getObject()), EventName + "_buffering", new Object[] { Integer.valueOf(percent)});
				}
			});  	
	    	
	  }
	 
	 
	 /**
	     * Loads a video file and prepares it for playing.
	     * It is not possible to load files from the assets folder.
	     * You can pass "http" to the Dir parameter and then a full URL (including http) to the FileName.
	     * In this case the online video will be streamed. If you want for instance stream using MMS,
	     * you must still pass "http" to the Dir parameter. 
	     * Note: There is no need to add the INTERNET permission for this to work since it is already 
	     * included in the B4A wrapper-library.
	     * Note2: No need to use the method Play - the streaming will start anyway.
	     * Note3: Depending on the URL, you may or not need to URL-encode before passing on the URL.
	     */	 
	  public void LoadVideo(String Dir, String FileName)
	  {
		  if (Dir.equals(anywheresoftware.b4a.objects.streams.File.getDirAssets())) {
	      throw new RuntimeException("Cannot load video from assets folder.");
	    }
	    if (Dir.equals("ContentDir")) {
	      vitamioExt.setVideoPath(FileName);
	    }
	    else if (Dir.equals("http")) {
	    	Uri uri = Uri.parse(FileName.toString());
	    	vitamioExt.setVideoURI(uri);
	   	    		    }
	    else
	    	vitamioExt.setVideoPath(new java.io.File(Dir, FileName).toString());
	  }

	  /**
	     * Starts or resumes playing.
	     */
	  public void Play()
	  {
		vitamioExt.start();
	  }
	  /**
	     * Pauses the playback.
	     */
	  public void Pause()
	  {
		  vitamioExt.pause();
	  }
	  /**
	     * Stops the playback.
	     */
	  public void Stop()
	  {
		  vitamioExt.stopPlayback();
	  }
	  /**
	     * Resumes the playback. Perhaps same as Play (not tested).
	     */
	  public void Resume()
	  {
		  vitamioExt.resume();
	  }
	  /**
	     * Tests whether the video is currently playing.
	     */
	  public boolean IsPlaying()
	  {
	    return vitamioExt.isPlaying();
	  }
	  /**
	     * Gets or sets the playing position (in milliseconds). Not tested.
	     */
	  public long getPosition()
	  {
	    return vitamioExt.getCurrentPosition();
	  }
	  public void setPosition(int v) {
		  vitamioExt.seekTo(v);
	  }
		
	  /**
	   	* Gets current playing position (in milliseconds). 
	   	*/
	  public long getCurrentPosition()
	  {
		  return vitamioExt.getCurrentPosition();
	  }
	  	  
	  /**
	     * Gets the video duration in milliseconds.
	     */
	  public long getDuration()
	  {
	    return vitamioExt.getDuration();
	  }
	  
	  /**
	     * Seeks to specified time position.
	     * Parameters:
	     * msec - the offset in milliseconds from the start to seek to
	     * Throws:
	     * java.lang.IllegalStateException - if the internal player engine has not been initialized
	     */
	  public void seekTo(long msec) throws java.lang.IllegalStateException
	  {
		  vitamioExt.seekTo(msec);
	  }

	  /**
	     * verifies if the media can seek backward.
	     */
	  public boolean canSeekBackward()
	  {
	    return vitamioExt.canSeekBackward();
	  }
	  
	  /**
	     * verifies if the media can seek forward.
	     */
	  public boolean canSeekForward()
	  {
	    return vitamioExt.canSeekForward();
	  }
 
	  
	  /**
	     * Returns the aspect ratio of the video.
	     * the aspect ratio of the video, or 0 if there is no video, 
	     * or the width and height is not available
	     */
	  public float getVideoAspectRatio()
	  {
	    return vitamioExt.getVideoAspectRatio();
	  }
	  /**
	     * Returns the width of the video.
	     */
	  public int getVideoWidth()
	  {
	    return vitamioExt.getVideoWidth();
	  }
	  /**
	     * Returns the height of the video.
	     */
	  public int getVideoHeight()
	  {
	    return vitamioExt.getVideoHeight();
	  }
	  /**
	     * Checks (what??) is valid or not. To test.
	     */
	  public boolean isValid()
	  {
	    return vitamioExt.isValid();
	  }
	  /**
	     * Checks whether the buffer is filled or not.
	     */
	  public boolean isBuffering()
	  {
	    return vitamioExt.isBuffering();
	  }
	  /**
	     * The buffer to fill before playback, default is 1024KB.
	     */
	  public void setBufferSize(int bufSize)
	  {
		  vitamioExt.setBufferSize(bufSize);
	  }
	  /**
	     * returns buffer percentage.
	     */
	  public int getBufferPercentage()
	  {
		  return vitamioExt.getBufferPercentage(); 
	  }
	  /**
	     *  sets the volume. No idea about ranges to use (perhaps 1 to 15). To test.
	     *  Might be better to use Phone-library in B4A.
	     */
	  public void setVolume(float leftVolume,float rightVolume)
	  {
		  vitamioExt.setVolume(leftVolume, rightVolume);
	  }
	  /**
	     * Sets whether the media controller is enabled. It is disabled by default.
	     * Note that the media player gets attached to the VideoView parent.
	     */
	  public void setMediaControllerEnabled(boolean v)
	  {
		  vitamioExt.setMediaController(v ? new MediaController(ba.context) : null);
		  vitamioExt.requestFocus();
	  }

	  public String toString() {
	    if (getObjectOrNull() == null)
	      return super.toString();
	    if (!vitamioExt.isPlaying()) {
	      return "Not playing";
	    }
	    return "Playing, Position=" + getPosition() + ", Duration=" + getDuration();
	  }
	  /**
	     * Returns current video-layout. This is not an official API.
	     */
	    public int getCurrentLayout()
		  {
		    return mLayout;
		  }
	   
	   /**
	     *  Set the display options. Default is VIDEO_LAYOUT_ZOOM or 3. 
	     *  Layout-parameters: 
	     *  VIDEO_LAYOUT_ORIGIN or 0 
	     *  VIDEO_LAYOUT_SCALE or 1 
	     *  VIDEO_LAYOUT_STRETCH or 2 
	     *  VIDEO_LAYOUT_ZOOM or 3 
	     *  aspectRatio - video aspect ratio, will auto detect if 0.
	     */
	    public void VideoLayout(int vLayout,float vAspectRatio)
	    {if (vLayout==0) {
	    	mLayout = VideoView.VIDEO_LAYOUT_ORIGIN;
		    }
		    else if (vLayout==1)  {
		    	mLayout = VideoView.VIDEO_LAYOUT_SCALE;
		    }
		    else if (vLayout==2)  {
		    	mLayout = VideoView.VIDEO_LAYOUT_STRETCH;
		    }
		    else if (vLayout==3)  {
		    	mLayout = VideoView.VIDEO_LAYOUT_ZOOM;
		    }
		    else
		    	mLayout = VideoView.VIDEO_LAYOUT_ORIGIN;
	    	if (vitamioExt != null)
	    	   vitamioExt.setVideoLayout(mLayout,0); 
	    	}  
	   
    	    
	    /**
	     * Set the quality when playing video. If there is too much lag, try VIDEOQUALITY_LOW.
	     * If there is too much pixeling, set the value to VIDEOQUALITY_HIGH
	     * I would suggest always to use VIDEOQUALITY_HIGH or VIDEOQUALITY_MEDIUM
	     * (most modern android-devices can cope with higher settings)
	     * Note that default value is VIDEOQUALITY_LOW.
	     * Quality-Parameters:
	     * VIDEOQUALITY_HIGH or 16
	     * VIDEOQUALITY_MEDIUM or 0
	     * VIDEOQUALITY_LOW or -16
	     */
	    public void VideoQuality(int vQuality)
	    {if (vQuality==16) {
	    	vidQuality=16;
		    }
		    else if (vQuality==-16)  {
		    	vidQuality=-16;
		    }
		    else if (vQuality==0)  {
		    	vidQuality=0;
		    }
		    else
		    	vidQuality=-16;
	    vitamioExt.setVideoQuality(vidQuality);
	     }  
	
	    /**
	     * Get the audio track number in playback
	     */
	    public int obtainAudioTrack()
		  {
		    return vitamioExt.getAudioTrack();
		  }
	    
	    /**
	     * Get the audio tracks embedded in the movie (e.g. "English", 3).
	     * Parameters:
	     * encoding - the encoding to format String, will detect it if null
	     * Returns:
	     * Map of track name and track number
	     */
	    public java.util.HashMap<java.lang.String,java.lang.Integer> getAudioTrackMap(java.lang.String encoding)
	    {
		    return vitamioExt.getAudioTrackMap(encoding);
		  }
	    
	    /**
	     * Set the audio track to play, must be in the result of getAudioTrackMap(String)
	     * Parameters:
	     * audioIndex - the track number
	     */
	    public void selectAudioTrack(int audioIndex)
		  {
		    vitamioExt.setAudioTrack(audioIndex);
		  }
	    
	    
	    
}
